<?php
/*
Plugin Name: Permalink Validator
Plugin URI: http://wordpress.org/extend/plugins/permalink-validator/
Description: Validates the URL used and if not matching the official permalink then it issues a HTTP 301 or HTTP 404 message.
Author: Rolf Kristensen
Version: 0.3
Author URI: http://smallvoid.com/
*/

function activate_validator()
{
	if (defined('BBPATH'))
		return false;
	if ( is_404() )
		return false;
	if ( is_preview() )
		return false;
	if ( is_search() )
		return false;	// use noindex in header
	if ( is_date() )
		return false;	// use noindex in header
	if ( is_trackback() )
		return false;
	if ( is_admin() )
		return false;
	if ( is_feed() )
		return false;	// block feeds from being index'ed in robots.txt
	return true;
}

function permalink_validator() {
	global $wp_query;
	global $wp_rewrite;
	global $posts;
	
	if (!activate_validator())
		return;

	if (!$wp_rewrite->using_permalinks())
	{
		if (isset($_SERVER['PATH_INFO']) && !empty($_SERVER['PATH_INFO']) && $_SERVER['PATH_INFO']!='/')
		{
			$wp_query->set_404();
			return;
		}
	}
	
	if ( is_home() )
	{
		$page = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		if (is_paged())
		{
			if (count($posts) < 1 || $posts[0]->ID=='')
			{
				$wp_query->set_404();
				return;
			}

			$paged = abs($wp_query->get('paged'));
			$link = get_settings('home') . '/';
			if ($wp_rewrite->using_index_permalinks())
				$link .= 'index.php/';
			if ($paged > 1)
				$link = $link . 'page/' . $paged . '/';
			
			if ($link != $page)
			{
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
		}
		else
		{
			if (!empty($_SERVER['QUERY_STRING']) || !empty($_SERVER['PATH_INFO']) || ($page!=get_settings('home') && $page!=(get_settings('home') . '/index.php')))
			{
				$link = get_settings('home');
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
		}
	}
	else if ( is_category() )
	{
		$cat_obj = $wp_query->get_queried_object();
		if (!isset($cat_obj))
		{
			$wp_query->set_404();
			return;
		}
		
		if (is_paged())
		{
			if (count($posts) < 1 || $posts[0]->ID=='')
			{
				$wp_query->set_404();
				return;
			}
		}
		
		if (!$wp_rewrite->using_permalinks())
		{
			$query = 'cat=' . $cat_obj->cat_ID;
			$paged = abs($wp_query->get('paged'));
			if (is_paged() && $paged > 1)
				$query = $query . '&paged=' . $paged;
		
			if ($_SERVER['QUERY_STRING']!=$query)
			{
				$link = get_category_link($cat_obj->cat_ID);
				if ($paged > 1)
					$link = $link . '&paged=' . $paged;
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
		}
		else
		{
			$page = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
			if (is_paged())
			{
				$paged = abs($wp_query->get('paged'));
				$link = get_category_link($cat_obj->cat_ID);
				if ($paged > 1)
					$link = $link . 'page/' . $paged . '/';
				
				if ($link != $page)
				{
					header('HTTP/1.1 301 Moved Permanently');
					header('Status: 301 Moved Permanently');
					header("Location: $link");
					exit(0);
				}
			}
			else
			if (get_category_link($cat_obj->cat_ID)!=$page)
			{
				$link = get_category_link($cat_obj->cat_ID);
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
		}
	}
	else if ( is_single() )
	{
		if (count($posts) < 1 || $posts[0]->ID=='')
		{
			$wp_query->set_404();
			return;
		}

		if (!$wp_rewrite->using_permalinks())
		{
			$query = 'p=' . $posts[0]->ID;
			if ($_SERVER['QUERY_STRING']!=$query)
			{
				$link = get_permalink($posts[0]->ID);
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}		
		}
		else
		{
			$page = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
			if (get_permalink($posts[0]->ID)!=$page)
			{
				$link = get_permalink($posts[0]->ID);
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
		}
	}
	else if ( is_page() )
	{
		if (count($posts) < 1 || $posts[0]->ID=='')
		{
			$wp_query->set_404();
			return;
		}

		if (get_option('show_on_front')=='page' && get_option('page_on_front')==$posts[0]->ID)
		{
			// Static frontpage (WP 2.1+)
			$page = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
			if ($page!=get_settings('home') && $page!=(get_settings('home') . '/index.php'))
			{
				$link = get_settings('home');
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
			return;
		}

		if (!$wp_rewrite->using_permalinks())
		{
			$query = 'page_id=' . $posts[0]->ID;
			if ($_SERVER['QUERY_STRING']!=$query)
			{
				$link = get_page_link($posts[0]->ID);
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}			
		}
		else
		{
			$page = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
			if (get_page_link($posts[0]->ID)!=$page)
			{
				$link = get_page_link($posts[0]->ID);
				header('HTTP/1.1 301 Moved Permanently');
				header('Status: 301 Moved Permanently');
				header("Location: $link");
				exit(0);
			}
		}
	}
	else
	{
		return;	// Unknown page-type (just pass it through)

		echo $_SERVER['REQUEST_URI'];
		exit(0);

		$wp_query->set_404();
		return;
	}
	return;
}

function fix_request_uri_iis()
{
	global $is_IIS;
	global $wp_rewrite;

	// Only IIS is having problem with REQUEST_URI
	if (!$is_IIS)
		return;

	// Should only fix REQUEST_URI for pages where the validator is used
	if (!activate_validator())
		return;

	if ($wp_rewrite->using_index_permalinks())
	{
		$_SERVER['REQUEST_URI'] = $_SERVER['SCRIPT_NAME'] . $_SERVER['PATH_INFO'];
	}
	else if ($wp_rewrite->using_permalinks())
	{
		// SCRIPT_NAME includes script-path + script-filename (remove the filename) and append request-path (PATH_INFO)
		$_SERVER['REQUEST_URI'] = substr($_SERVER['SCRIPT_NAME'], 0, strrpos($_SERVER['SCRIPT_NAME'], '/')) . $_SERVER['PATH_INFO'];
	}
	else
	{
		$_SERVER['REQUEST_URI'] = $_SERVER['SCRIPT_NAME'];
	}

	// Append the query string if it exists and isn't null
	if (isset($_SERVER['QUERY_STRING']) && !empty($_SERVER['QUERY_STRING']))
	{
		$_SERVER['REQUEST_URI'] .= '?' . $_SERVER['QUERY_STRING'];
	}
}

// Fix for missing slash in category- and page-links when not having trailing slash in post permalink structure (WP 2.2+)
function fix_user_trailingslashit($link, $type) {
	global $wp_rewrite;
	if ($wp_rewrite->using_permalinks() && $wp_rewrite->use_trailing_slashes==false && $type != 'single')
		return trailingslashit($link);
	return $link;
}

if ( function_exists( 'add_filter' ) && function_exists('user_trailingslashit'))
{
	add_filter('user_trailingslashit', 'fix_user_trailingslashit', 66, 2 );
}

if ( function_exists( 'add_action' ) )
{
	add_action('template_redirect', 'permalink_validator');
	add_action('init', 'fix_request_uri_iis');
}
?>